/*
 * byteswap.h -- macros for converting between big and little endian
 */

#ifndef _byteswap_h
#define _byteswap_h

#include <endian.h>
#if __BYTE_ORDER != __LITTLE_ENDIAN

/* undef macros from <linux/msdos_fs.h> */
#undef CF_LW_W
#undef CF_LW_L
#undef CT_LW_W
#undef CT_LW_L

/* new definitions */
#define CF_LE_W(v) le16_to_cpu(v)
#define CF_LE_L(v) le32_to_cpu(v)
#define CT_LE_W(v) cpu_to_le16(v)
#define CT_LE_L(v) cpu_to_le32(v)

/* now the conversions proper;
 * code partially taken from <asm-m68k/byteorder.h>
 */
#define le16_to_cpu(x) \
	(__builtin_constant_p(x) ? __constant_swab16(x) : __swab16(x))
#define cpu_to_le16(x) \
	(__builtin_constant_p(x) ? __constant_swab16(x) : __swab16(x))
#define le32_to_cpu(x) \
	(__builtin_constant_p(x) ? __constant_swab32(x) : __swab32(x))
#define cpu_to_le32(x) \
	(__builtin_constant_p(x) ? __constant_swab32(x) : __swab32(x))

extern __inline__ unsigned short __constant_swab16( unsigned short val )
{
	return( (val << 8) | (val >> 8) );
}

extern __inline__ unsigned short __swab16( unsigned short val )
{
#ifdef __mc68000__
	/* fast assembler version for m68k */
	__asm__( "rolw #8,%0" : "=d" (val) : "0" (val) );
#else
	val = __constant_swab16( val );
#endif
	return( val );
}

extern __inline__ unsigned int __constant_swab32( unsigned int val )
{
	return( (val << 24) | ((val << 8) & 0xff0000) |
			((val >> 8) & 0xff00) | (val >> 24) );
}

extern __inline__ unsigned int __swab32( unsigned int val )
{
#ifdef __mc68000__
	/* fast assembler version for m68k */
	__asm__( "rolw #8,%0; swap %0; rolw #8,%0" : "=d" (val) : "0" (val) );
#else
	val = __constant_swab32( val );
#endif
	return( val );
}

#else /* __BYTE_ORDER != __LITTLE_ENDIAN */

/* undef macros from <linux/msdos_fs.h> */
#undef CF_LW_W
#undef CF_LW_L
#undef CT_LW_W
#undef CT_LW_L

/* new definitions */
#define CF_LE_W(v) (v)
#define CF_LE_L(v) (v)
#define CT_LE_W(v) (v)
#define CT_LE_L(v) (v)

#endif /* __BYTE_ORDER != __LITTLE_ENDIAN */

#endif  /* _byteswap_h */

